/**
 * @file detail.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Jan-2013 / All rights reserved. 
 * ---------------------------------------------------------
 * Provides gui content (TAB: Detail)
**/
#include "detail.h"
#include "dmxgui_tools.h"
#include "listing.h"
#include "../dmxplayer.h"
#include "../dmxstr.h"
#include <string.h>

#define PLAY_MENU_MAX_X 	1
#define PLAY_MENU_MAX_Y 	3

#define PLAY_MENU_NAME      (play_menu_y == -1)
#define PLAY_MENU_FADE      ((play_menu_x == 0) && (play_menu_y == 0))
#define PLAY_MENU_LENGTH    ((play_menu_x == 1) && (play_menu_y == 0))
#define PLAY_MENU_STATUS    ((play_menu_x == 0) && (play_menu_y == 1))
#define PLAY_MENU_TRIGGER   ((play_menu_x == 1) && (play_menu_y == 1))
#define PLAY_MENU_GROUP     ((play_menu_x == 0) && (play_menu_y == 2))
#define PLAY_MENU_SPEED     ((play_menu_x == 1) && (play_menu_y == 2))
#define PLAY_MENU_BACKWARD  ((play_menu_x == 0) && (play_menu_y == 3))
#define PLAY_MENU_FORWARD   ((play_menu_x == 1) && (play_menu_y == 3))

int play_menu_x   =  0;
int play_menu_y   = -1;

/* A key event in active play mode */
void detail_keyevent(int c, dmxgui_status_t *gui)
{
	int sceneview;
	dmxprogram_t *p;
	sceneview = listing_getSelectedProgram();
	p = dmxplayer_getprogram(sceneview);
	switch(c)
	{
		/* toogle menu position */
		case KEY_LEFT:
			if(--play_menu_x<0) play_menu_x = 0;
			break;
		case KEY_RIGHT:
			/* in step selector */
			if(play_menu_y > PLAY_MENU_MAX_Y)
			{
			}
			/* in top settings */
			if(++play_menu_x>PLAY_MENU_MAX_X) play_menu_x = PLAY_MENU_MAX_X;
			break;
		case KEY_UP:
			if(--play_menu_y<-1) play_menu_y = -1;
			break;
		case KEY_DOWN:
			if(++play_menu_y>(PLAY_MENU_MAX_Y+p->size)) play_menu_y = PLAY_MENU_MAX_Y;
			break;
		/* increase command */
		case '+':
			if(p==0) break;
			if(PLAY_MENU_FADE)
			{
				if( ++(p->fade) > 100) p->fade = 100;
			}
			else if(PLAY_MENU_GROUP)
			{
				if( ++(p->group) > 20) p->group = 20;
			}
			else if(PLAY_MENU_TRIGGER)
			{
				if( ++(p->trigger) > 1) p->trigger = 0;
			}
			else if(PLAY_MENU_SPEED)
			{
				dmxprogram_speed_decrease(p);
			}
			else if( PLAY_MENU_STATUS )
			{
				dmxplayer_toogle_program(sceneview);
			}
			break;
		/* decrease command */
		case '-':
			if(p==0) break;
			if(PLAY_MENU_FADE)
			{
				if( --(p->fade) < 0) p->fade = 0;
			}
			else if(PLAY_MENU_GROUP)
			{
				if( --(p->group) < 0) p->group = 0;
			}
			else if(PLAY_MENU_TRIGGER)
			{
				if( --(p->trigger) < 0) p->trigger = 1;
			}
			else if(PLAY_MENU_SPEED)
			{
				dmxprogram_speed_increase(p);
			}
			else if( PLAY_MENU_STATUS )
			{
				dmxplayer_toogle_program(sceneview);
			}
			break;
		/* enter */
		case KEY_ENTER:
		case '\r':
		case '\n':
			if(PLAY_MENU_BACKWARD && (p->trigger == 0))
			{
				if( --(p->pos) < 0) p->pos = p->size-1;
				p->refresh++;
			}
			else if(PLAY_MENU_FORWARD && (p->trigger == 0))
			{
				if( ++(p->pos) >= p->size) p->pos = 0;
				p->refresh++;
			}
			else if(PLAY_MENU_NAME)
			{
				gui->is_user_input = 1;
			}
			break;
		/* user input finished */
		case DMXGUI_KEY_INPUT_FIN:
			if(PLAY_MENU_NAME)
			{
				dmxprogram_set_name(p, gui->input);
				gui->input[0] = '\0';
				gui->inplen = 0;
			}
			break;
	}
}


/* Print active play window */
void detail_printer(WINDOW* wnd, dmxgui_status_t *gui)
{
	dmxprogram_t *p;
	int line, sceneview;
	float fadetime;
	char strbuf[12];
	/* get selected  program */
	sceneview = listing_getSelectedProgram();

	/* draw window frame */
	wbkgd(wnd, COLOR_PAIR(1));
	box(wnd, 0, 0);

	/* Get program and print menu */
	p = dmxplayer_getprogram(sceneview);
	line = 6;

	wattron(wnd, COLOR_PAIR(2));
		/* show key and name */
		mvwhline(wnd, 3, 2, ' ', COLS-3);
		if(PLAY_MENU_NAME)
		{
			wattron(wnd, A_REVERSE); 
				if(gui->is_user_input) mvwprintw(wnd, 3, 2, "Chaser details [%02d] >%s< ", sceneview+1, gui->input);
				else                   mvwprintw(wnd, 3, 2, "Chaser details [%02d] %s ", sceneview+1, p->name);
			wattroff(wnd, A_REVERSE); 
		} else mvwprintw(wnd, 3, 2, "Chaser details [%02d]: %s", sceneview+1, p->name);
		mvwprintw(wnd, 4, 2, "(Press enter to change)");

		/* fade value */
		if(p->steplen >= 0) fadetime = (p->fade * dmxplayer_get_speed_sec() * p->steplen) / 100;
		else fadetime = (-1) * p->fade * dmxplayer_get_speed_sec() / (p->steplen*100);
		if( PLAY_MENU_FADE )
		{
		
			wattron(wnd, A_REVERSE); 
				mvwprintw(wnd, line, 2,  "Fade Time: %3d%% (%4.2fs)    ", p->fade, fadetime);
			wattroff(wnd, A_REVERSE); 
		} else mvwprintw(wnd, line, 2, "Fade Time: %3d%% (%4.2fs)    ", p->fade, fadetime);
		/* length */
		if( PLAY_MENU_LENGTH )
		{
		
			wattron(wnd, A_REVERSE); 
				mvwprintw(wnd, line, COLS/2+2,  "Length:  %03d steps  ", p->size);
			wattroff(wnd, A_REVERSE); 
		} else mvwprintw(wnd, line, COLS/2+2, "Length:  %03d steps  ", p->size);

		/* is active */
		line=line+2;
		if( PLAY_MENU_STATUS )
		{
		
			wattron(wnd, A_REVERSE); 
				if(p->active) mvwprintw(wnd, line, 2, "Status: Running ");
				else          mvwprintw(wnd, line, 2, "Status: Stop    ");
			wattroff(wnd, A_REVERSE); 
		} else {
			if(p->active) mvwprintw(wnd, line, 2, "Status: Running ");
			else          mvwprintw(wnd, line, 2, "Status: Stop    ");
		}
		/* trigger mode */
		if( PLAY_MENU_TRIGGER )
		{
			wattron(wnd, A_REVERSE); 
				if(p->trigger==0)      mvwprintw(wnd, line, COLS/2+2, "Trigger: Sequence   ");
				else if(p->trigger==1) mvwprintw(wnd, line, COLS/2+2, "Trigger: Beat       ");
			wattroff(wnd, A_REVERSE); 
		} else {
			if(p->trigger==0)      mvwprintw(wnd, line, COLS/2+2, "Trigger: Sequence   ");
			else if(p->trigger==1) mvwprintw(wnd, line, COLS/2+2, "Trigger: Beat       ");
		}

		/* ? */
		line=line+2;
		if( PLAY_MENU_GROUP )
		{
		
			wattron(wnd, A_REVERSE); 
				if(p->group != 0) mvwprintw(wnd, line, 2,  "Group:  %02d      ", p->group);
				else mvwprintw(wnd, line, 2,  "Group:  none    ", p->group);
			wattroff(wnd, A_REVERSE); 
		} else {
			if(p->group != 0) mvwprintw(wnd, line, 2,  "Group:  %02d      ", p->group);
			else mvwprintw(wnd, line, 2,  "Group:  none    ", p->group);
		}
		/* speed */
		dmxstr_translate_steplen(strbuf, p->steplen);
		if( PLAY_MENU_SPEED )
		{
			wattron(wnd, A_REVERSE); 
				mvwprintw(wnd, line, COLS/2+2,  "Measure:   %s          ", strbuf);
			wattroff(wnd, A_REVERSE); 
		} else mvwprintw(wnd, line, COLS/2+2, "Measure:   %s          ", strbuf);


		/* STEP LISTING */
		line=line+3;
		mvwprintw(wnd, line++, 2, "Step listing (Name, Trigger Type, Trigger Unit, Fade in):");

		/* print steps */
		detail_print_steplist(wnd, line, 2, 0);

	wattroff(wnd, COLOR_PAIR(2));
	
	line=line+2;

	/* show information */	
	dmxgui_print_key_bindings(wnd);
}



void detail_print_steplist(WINDOW *wnd, int y, int x, long beginat)
{
	int n, sceneview;
	dmxprogram_t *p;
	sceneview = listing_getSelectedProgram();
	//maxlen = COLS-7;
	/* get program */
	p = dmxplayer_getprogram(sceneview);
	if(p==0) return;
	if(p->size == 0) return;
	/* step iterator */
	for(n=0; n<p->size; n++)
	{
		mvwprintw(wnd, y, x, "   Step %d    ", n);
		if(p->data[n]->trigger_type == DMXCUE_TRIGGER_ONSTEP)
			mvwprintw(wnd, y, x+15, "Beat  ");
		else if(p->data[n]->trigger_type == DMXCUE_TRIGGER_ONTIME)
			mvwprintw(wnd, y, x+15, "Time  ");
		else if(p->data[n]->trigger_type == DMXCUE_TRIGGER_ONSIGNAL)
			mvwprintw(wnd, y, x+15, "Manual");
		else
			mvwprintw(wnd, y, x+15, "?     ");
		mvwprintw(wnd, y, x+30, "time:%d    ", p->data[n]->trigger_time);
		mvwprintw(wnd, y, x+45, "fade:%d    ", p->data[n]->trigger_fade);
		if( n == p->pos )
			mvwprintw(wnd, y, x, "->");
		y++;
	}
}


