/**
 * @file edit.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Jan-2013 / All rights reserved. 
 * ---------------------------------------------------------
 * Provides gui content
**/
#include "edit.h"
#include "listing.h"
#include "dmxgui_tools.h"
#include "../dmxplayer.h"

int edit_cue = 1;
char e_input[10];
int  e_input_cnt = 0;
int s_channel = 1;

/* A key event in active play mode */
void edit_keyevent(int c, dmxgui_status_t *gui)
{
	int tmp;
	int global_selected = listing_getSelectedProgram();
	dmxprogram_t *p = 0;
	dmxlist_t *cue  = 0;
	p = dmxplayer_getprogram(global_selected);
	if(p != 0) cue = dmxprogram_get_cue((edit_cue-1), p);
	if( (c >= '0') && (c <= '9') )
	{
		if(e_input_cnt < 3)
		{
			e_input[e_input_cnt]   = c;
			e_input[++e_input_cnt] = '\0';
		}
	}
	else
	{
		e_input_cnt=0;
		switch(c)
		{
			/* toogle channel */
			case KEY_LEFT:
				s_channel--;
				if(s_channel < 0) s_channel = 0;
				break;
			case KEY_RIGHT:
				s_channel++;
				if(s_channel >= (gui->channel_cnt)) s_channel = (gui->channel_cnt) - 1;
				break;
			case KEY_UP:
				s_channel -= (COLS-3)/5;
				if(s_channel < 0) s_channel = 0;
				break;
			case KEY_DOWN:
				s_channel += (COLS-3)/5;
				if(s_channel >= (gui->channel_cnt)) s_channel = (gui->channel_cnt) - 1;
				break;
			/* unset channel */
			case KEY_F(5):
			case 'U':
				if(cue == 0) break;
				dmxcue_unset(s_channel, cue);
				break;
			/* clear cue */
			case KEY_F(6):
			case 'C':
				if(cue == 0) break;
				dmxcue_clear(cue);
				break;
			/* toogle channel */
			case KEY_ENTER:
			case '\r':
			case '\n':
				if(e_input[0] == '\0') break;
				tmp = atoi(e_input);
				if(tmp > 255) tmp = 255;
				if(tmp < 0)   tmp = 0;
				e_input[0] = '\0';
				dmxcue_set(s_channel, tmp, cue);
				break;
			/* +/- 1 */
			case '+':
				dmxcue_setadd(s_channel,  1, cue);
				break;
			case '-':
				dmxcue_setadd(s_channel, -1, cue);
				break;
			/* +/- 10 */
			case KEY_PPAGE:
				dmxcue_setadd(s_channel,  10, cue);
				break;
			case KEY_NPAGE:
				dmxcue_setadd(s_channel, -10, cue);
				break;
			/* toogle next cue */
			case ',':
				if(--edit_cue < 1) edit_cue = 1;
				break;
			case '.':
				if(++edit_cue >= p->size) edit_cue = p->size;
				break;
			/* toogle first cue */
			case KEY_HOME:
				edit_cue = 1;
				break;
			/* toogle last cue */
			case KEY_END:
				edit_cue = p->size;
				break;
		}
	}
}


/* Print active play window */
void edit_printer(WINDOW *wnd, dmxgui_status_t *gui)
{
	int x, y, i;
	dmxprogram_t *p;
	int global_selected = listing_getSelectedProgram();
	/* draw window frame */
	wbkgd(wnd, COLOR_PAIR(1));
	box(wnd, 0, 0);

	if(e_input_cnt == 0) e_input[0] = '\0';

	/* draw window content */
	if( (p = dmxplayer_getprogram(global_selected)) && (p->size != 0) )
	{
		/* check values */
		if(edit_cue < 1) edit_cue = 1;
		if(edit_cue >= p->size) edit_cue = p->size;
		/* show edit info */
		mvwprintw(wnd, 3, 2, "Edit Chaser: %s [at cue %d]  -  ,/. to toogle", p->name, edit_cue);
		
		/* iterate all channels */
		dmxpair_t *data;
		dmxlist_t *cue = dmxprogram_get_cue((edit_cue-1), p);
		x = 2; y = 5;
		wattron(wnd, COLOR_PAIR(2));
		for(i=0; i<(gui->channel_cnt); ++i)
		{	
			data = dmxcue_get_channel_obj(i, cue);
			/* selected channel */
			if(s_channel == i)
			{
				wattron(wnd, A_REVERSE); 
					mvwprintw(wnd, y, x, " %3d ", i+1);
					if(e_input_cnt == 0)
					{
						if(data!=0) mvwprintw(wnd, y+1, x, " %03d ", data->value);
						else      mvwprintw(wnd, y+1, x, " --- ");
					}
					else
					{					
						wattroff(wnd, COLOR_PAIR(2)); wattron(wnd, COLOR_PAIR(4));
							mvwprintw(wnd, y+1, x, ">%03d<", atoi(e_input));
						wattroff(wnd, COLOR_PAIR(4)); wattron(wnd, COLOR_PAIR(2));
					}
				wattroff(wnd, A_REVERSE);
			}
			/* other channels */
			else
			{
				mvwprintw(wnd, y, x, " %3d ", i+1);
				if(data!=0) mvwprintw(wnd, y+1, x, " %03d ", data->value);
				else       mvwprintw(wnd, y+1, x, " --- ");
			}
			/* calculate next position */
			x += 5;
			if( (x+5) >= COLS) 
			{
				x = 2;
				y += 3;
			}
		}
		wattroff(wnd, COLOR_PAIR(2));
	}
	else mvwprintw(wnd, 3, 2, "Empty chaser");
	/* show information */	
	dmxgui_print_key_bindings(wnd);
}
